@@ -1,27 +1,23 @@
 //! Various assertions.
 
 /// Asserts that two accounts share the same key.
+///
+/// Deprecated in favor of [assert_keys_eq].
+#[deprecated]
 #[macro_export]
 macro_rules! assert_keys {
     ($account_a: expr, $account_b: expr $(,)?) => {
-        assert_keys!($account_a, $account_b, "key mismatch")
+        assert_keys_eq!($account_a, $account_b, "key mismatch")
     };
     ($account_a: expr, $account_b: expr, $msg: expr $(,)?) => {
-        let __account_a = anchor_lang::Key::key(&$account_a);
-        let __account_b = anchor_lang::Key::key(&$account_b);
-        if __account_a != __account_b {
-            msg!(
-                "Key mismatch: {}: {} (left) != {} (right)",
-                $msg,
-                __account_a,
-                __account_b
-            );
-            return Err($crate::VipersError::KeyMismatch.into());
-        }
+        assert_keys_eq!($account_a, $account_b, $msg)
     };
 }
 
 /// Asserts that the ATA is the one of the given owner/mint.
+///
+/// Warning: this uses a lot of compute units due to the need to generate a PDA.
+/// It is recommended to cache this value.
 #[macro_export]
 macro_rules! assert_ata {
     ($ata: expr, $owner: expr, $mint: expr $(,)?) => {
@@ -31,8 +27,7 @@ macro_rules! assert_ata {
         let __owner = anchor_lang::Key::key(&$owner);
         let __mint = anchor_lang::Key::key(&$mint);
         let __ata = anchor_lang::Key::key(&$ata);
-        let __real_ata =
-            $crate::spl_associated_token_account::get_associated_token_address(&__owner, &__mint);
+        let __real_ata = $crate::ata::get_associated_token_address(&__owner, &__mint);
         if __real_ata != __ata {
             msg!(
                 "ATA mismatch: {}: {} (left) != {} (right)",
@@ -48,6 +43,9 @@ macro_rules! assert_ata {
 }
 
 /// Asserts that the given [anchor_spl::token::TokenAccount] is an associated token account.
+///
+/// Warning: this uses a lot of compute units due to the need to generate a PDA.
+/// Use this macro sparingly.
 #[macro_export]
 macro_rules! assert_is_ata {
     ($ata: expr $(,)?) => {
@@ -75,6 +73,9 @@ macro_rules! assert_is_ata {
 }
 
 /// Asserts that an account is owned by the given program.
+///
+/// As of Anchor 0.15, Anchor handles this for you automatically.
+/// You should not need to use this.
 #[macro_export]
 macro_rules! assert_owner {
     ($program_account: expr, $owner: expr $(,)?) => {
@@ -147,12 +148,7 @@ macro_rules! assert_keys_eq {
         );
     };
     ($account_a: expr, $account_b: expr, $err: expr $(,)?) => {
-        assert_keys_eq!(
-            $account_a,
-            $account_b,
-            $err,
-            &*format!("{:?}: {}", $err, $err)
-        );
+        assert_keys_eq!($account_a, $account_b, $err, format_err!($err));
     };
     ($account_a: expr, $account_b: expr, $err: expr, $msg: expr $(,)?) => {
         let __account_a = $account_a.key();
@@ -160,11 +156,9 @@ macro_rules! assert_keys_eq {
         if __account_a != __account_b {
             msg!($msg);
             msg!(stringify!($account_a != $account_b));
-            msg!("Left:");
-            msg!("{}", __account_a);
-            msg!("Right:");
-            msg!("{}", __account_b);
-            return Err($err.into());
+            msg!("Left: {}", __account_a);
+            msg!("Right: {}", __account_b);
+            throw_err!($err);
         }
     };
 }
@@ -205,12 +199,7 @@ macro_rules! assert_keys_neq {
         );
     };
     ($account_a: expr, $account_b: expr, $err: expr $(,)?) => {
-        assert_keys_neq!(
-            $account_a,
-            $account_b,
-            $err,
-            &*format!("{:?}: {}", $err, $err)
-        );
+        assert_keys_neq!($account_a, $account_b, $err, format_err!($err));
     };
     ($account_a: expr, $account_b: expr, $err: expr, $msg: expr $(,)?) => {
         let __account_a = $account_a.key();
@@ -218,11 +207,9 @@ macro_rules! assert_keys_neq {
         if __account_a == __account_b {
             msg!($msg);
             msg!(stringify!($account_a == $account_b));
-            msg!("Left:");
-            msg!("{}", __account_a);
-            msg!("Right:");
-            msg!("{}", __account_b);
-            return Err($err.into());
+            msg!("Left: {}", __account_a);
+            msg!("Right: {}", __account_b);
+            throw_err!($err);
         }
     };
 }
@@ -265,7 +252,7 @@ macro_rules! unwrap_or_err {
 #[macro_export]
 macro_rules! unwrap_int {
     ($option:expr $(,)?) => {
-        $option.ok_or_else(|| -> ProgramError { $crate::VipersError::IntegerOverflow.into() })?
+        unwrap_opt!($option, $crate::VipersError::IntegerOverflow)
     };
 }
 
@@ -315,7 +302,43 @@ macro_rules! program_err {
     };
 }
 
+/// Throws an error.
+///
+/// # Example
+///
+/// ```
+/// # use anchor_lang::prelude::*;
+/// # impl From<ErrorCode> for ProgramError { fn from(code: ErrorCode) -> Self { ProgramError::Custom(10) } }
+/// # pub enum ErrorCode { MyError }
+/// # #[macro_use] extern crate vipers; fn main() -> ProgramResult {
+/// let fail = false;
+/// if fail {
+///     throw_err!(MyError);
+/// }
+/// Ok(())
+/// # }
+/// ```
+#[macro_export]
+macro_rules! throw_err {
+    ($error:ident $(,)?) => {
+        throw_err!(crate::ErrorCode::$error);
+    };
+    ($error:expr $(,)?) => {
+        log_code_location!();
+        return Err($error.into());
+    };
+}
+
+/// Logs where in the code the macro was invoked.
+#[macro_export]
+macro_rules! log_code_location {
+    () => {
+        msg!("Error thrown at {}:{}", file!(), line!());
+    };
+}
+
 /// Asserts that an invariant holds, otherwise logs the given message.
+/// This is a drop-in replacement for `require!`.
 ///
 /// # Example
 ///
@@ -327,14 +350,31 @@ macro_rules! program_err {
 /// ```
 #[macro_export]
 macro_rules! invariant {
+    ($invariant: expr, $err_code: ident $(,)?) => {
+        invariant!($invariant, crate::ErrorCode::$err_code);
+    };
+    ($invariant: expr, $msg: literal $(,)?) => {
+        invariant!($invariant, $crate::VipersError::InvariantFailed, $msg);
+    };
     ($invariant:expr, $err:expr $(,)?) => {
+        invariant!($invariant, $err, format_err!($err));
+    };
+    ($invariant:expr, $err:expr, $msg: expr $(,)?) => {
         if !($invariant) {
             msg!("Invariant failed: {:?}", $err);
-            return Err($crate::VipersError::InvariantFailed.into());
+            throw_err!($crate::VipersError::InvariantFailed);
         }
     };
 }
 
+/// Formats an error as a `&str`.
+#[macro_export]
+macro_rules! format_err {
+    ($err: expr) => {
+        &*format!("{:?}: {}", $err, $err)
+    };
+}
+
 /// Attempts to unwrap an [Option], and if it fails, prints an error.
 ///
 /// # Example
@@ -349,9 +389,19 @@ macro_rules! invariant {
 /// ```
 #[macro_export]
 macro_rules! unwrap_opt {
+    ($option: expr, $err_code: ident $(,)?) => {
+        unwrap_opt!($option, crate::ErrorCode::$err_code)
+    };
+    ($option: expr, $msg: literal $(,)?) => {
+        unwrap_opt!($option, $crate::VipersError::OptionUnwrapFailed, $msg)
+    };
     ($option:expr, $err:expr $(,)?) => {
+        unwrap_opt!($option, $err, format_err!($err))
+    };
+    ($option:expr, $err:expr, $msg: expr $(,)?) => {
         $option.ok_or_else(|| -> ProgramError {
             msg!("Option unwrap failed: {:?}", $err);
+            log_code_location!();
             $crate::VipersError::OptionUnwrapFailed.into()
         })?
     };
@@ -369,10 +419,16 @@ mod tests {
         pub byte: u8,
     }
 
+    #[allow(deprecated)]
     #[test]
-    fn test_compiles() -> ProgramResult {
+    fn test_compiles_deprecated() -> ProgramResult {
         assert_keys!(token::ID, token::ID, "token program");
 
+        Ok(())
+    }
+
+    #[test]
+    fn test_compiles() -> ProgramResult {
         assert_ata!(
             get_associated_token_address(&token::ID, &token::ID),
             token::ID,
